// src/index.ts
import path from "path";
import { fdir } from "fdir";
import picomatch from "picomatch";
function expandDir(pattern) {
  if (pattern.endsWith("/")) {
    return `${pattern}**`;
  }
  if (pattern.endsWith("\\")) {
    return `${pattern.slice(0, -1)}/**`;
  }
  return `${pattern}/**`;
}
function processPatterns({ patterns, ignore = [], expandDirectories = true }) {
  const matchPatterns = [];
  const ignorePatterns = ignore.map((p) => !p.endsWith("*") && expandDirectories ? expandDir(p) : p);
  if (!patterns || patterns.length === 0) {
    return { match: ["**/*"], ignore: ignorePatterns };
  }
  for (let pattern of patterns) {
    if (!pattern.endsWith("*") && expandDirectories) {
      pattern = expandDir(pattern);
    }
    if (pattern.startsWith("!") && pattern[1] !== "(") {
      ignorePatterns.push(pattern.slice(1));
    } else {
      matchPatterns.push(pattern);
    }
  }
  return { match: matchPatterns, ignore: ignorePatterns };
}
function getFdirBuilder(options, cwd) {
  const processed = processPatterns(options);
  const matcher = picomatch(processed.match, {
    dot: options.dot,
    ignore: processed.ignore
  });
  const fdirOptions = {
    // use relative paths in the matcher
    filters: [options.absolute ? (p) => matcher(p.slice(cwd.length + 1)) : matcher],
    relativePaths: true
  };
  if (options.deep) {
    fdirOptions.maxDepth = options.deep;
  }
  if (options.absolute) {
    fdirOptions.relativePaths = false;
    fdirOptions.resolvePaths = true;
    fdirOptions.includeBasePath = true;
  }
  if (options.onlyDirectories) {
    fdirOptions.excludeFiles = true;
    fdirOptions.includeDirs = true;
  } else if (options.onlyFiles === false) {
    fdirOptions.includeDirs = true;
  }
  return new fdir(fdirOptions);
}
async function glob(patternsOrOptions, options) {
  if (patternsOrOptions && (options == null ? void 0 : options.patterns)) {
    throw new Error("Cannot pass patterns as both an argument and an option");
  }
  const opts = Array.isArray(patternsOrOptions) ? { ...options, patterns: patternsOrOptions } : patternsOrOptions;
  const cwd = opts.cwd ? path.resolve(opts.cwd) : process.cwd();
  return getFdirBuilder(opts, cwd).crawl(cwd).withPromise();
}
function globSync(patternsOrOptions, options) {
  if (patternsOrOptions && (options == null ? void 0 : options.patterns)) {
    throw new Error("Cannot pass patterns as both an argument and an option");
  }
  const opts = Array.isArray(patternsOrOptions) ? { ...options, patterns: patternsOrOptions } : patternsOrOptions;
  const cwd = opts.cwd ? path.resolve(opts.cwd) : process.cwd();
  return getFdirBuilder(opts, cwd).crawl(cwd).sync();
}
export {
  glob,
  globSync
};
//# sourceMappingURL=index.mjs.map